==> data_cache-0.0.1%2B1.tar.gz: lib/cacher.dart <==

import 'dart:convert';
import 'dart:core';

import 'package:shared_preferences/shared_preferences.dart';

class cacheManager{
  static const inf="info";
  static const cas="cs";
  static const ts="ts";
  static const av="av";

  static const map="d";
  static const array="a";
  // static const lmt=50;
  static const array_N="d";
  static const array_ID="a";

  static const maxCommit=3;

  late Map<Type,dataCount> data;
  late SharedPreferences sp;
  Iterable<Type> get classesKeys=>converter.keys;
  final Map<Type,List<Function>> converter;

  int commits=0;
  cacheManager(this.converter){
    if(!converter.keys.contains(Map))converter[Map]=[(Map m)=>m,(Map m)=>m];
    SharedPreferences.getInstance().then((v){
      sp=v;
      classesKeys.forEach((c) {
        if(v.containsKey(c.toString())&&v.getString(c.toString())!=null) {
          data[c]=dataCount.fromMap(json.decode(v.getString(c.toString())!));
        }
      });
    });
    data=Map<Type,dataCount>.fromIterables(classesKeys,
        [for(var c in classesKeys)dataCount()]);
  }

  void incrementCommits(){
    commits++;
    if(commits==maxCommit){
      classesKeys.forEach((k) {
        if(data[k]!=null)
          sp.setString(k.toString(), json.encode(data[k]!.toMap()));});
      commits=0;
    }
  }

  void addOrUpdate<T>(T i){
    final Map<String,dynamic> r=converter[T]![0](i);
    data[T]!.insert(r["id"], r);
    incrementCommits();
  }
  void update(Type T,String key,Map<String,dynamic> i){
    data[T]!.modify(key, i);
    incrementCommits();
  }

  T? get<T>(String id){
    incrementCommits();
    return converter[T]![1](data[T]!.get(id));
  }


  void check(){
    data.forEach((key, value) {
      print(key);
      value.check();
    });
  }
}

class dataCount{
  static const lmt=6;
  static const int intMaxValue = 9007199254740991;
  int count=0;
  // dynamic data;
  final List<String> array;//=<String>["0"];
  final Map<String,keyCount> map;//=<String,keyCount<T>>{};
  dataCount():array=<String>["0"],map=<String,keyCount>{};

  void check(){
    print("  array");
    array.forEach((e) {
      print("    ${e}");
      print("    ${map[e].toString()}");
    });
    print("  map");
    map.forEach((key, value) {
      print("    key:${key}");
      print("    data:${value.toString()}");
    });
  }

  Map<String,dynamic> toMap(){
    final mapm={};
    map.forEach((k, v) {mapm[k]= v.toMap();});
    return {"array":array,"map":mapm};
  }

  dataCount.fromMap(Map<String,dynamic> m):
        array=m["array"].cast<String>(),
        map=(m["map"] as Map).map((k, v) => MapEntry(k, keyCount.formMap(v)));

  Map<String,dynamic>? get(String key){
    final r=map[key];
    if(r!=null) insert(key, r.data);
    return r?.data;
  }

  ///if(key already exist) count+1,also set data to newdata
  ///if(key not exist)
  ///   if(not full) insert keyCount(data: data,count: 1)
  ///   if(full) insert keyCount(data: data,count: minUseCount+1)
  void insert(String key,Map<String,dynamic> data){
    if(map.containsKey(key)){ //// add count by 1 then bubble down
      int i=1;
      while(array[i]!=key){i++;}
      map[key]!.add1();
      map[array[i]]!.data=data;
      bubbledown(i);
    }else if(array.length<lmt){ /// add to bottom then bubble up
      map[key]=keyCount(data: data);
      int i=array.length;
      array.add(key);
      bubbleup(i);
    }else{
      final oldKey=array[1];
      map[key]=keyCount(data: data,count: map[oldKey]!.count+1);
      array[1]=key;
      map.remove(oldKey);
      bubbledown(1);
    }
  }

  ///modify data,without adding count
  void modify(String key,Map<String,dynamic> change){
    if(map.containsKey(key)){
      map[key]!.data=map[key]!.data..addAll(change);
    }
  }

  bubbledown(int i){
    int l=((lchild(i)?.count)??(intMaxValue));
    int r=((rchild(i)?.count)??(intMaxValue));
    while(map[array[i]]!.count>l||map[array[i]]!.count>r){
      if(r>l){
        swap(i, i*2);
        i=i*2;
      }
      else{
        swap(i, i*2+1);
        i=i*2+1;
      }
      l=((lchild(i)?.count)??(intMaxValue));
      r=((rchild(i)?.count)??(intMaxValue));
    }
  }

  bubbleup(int i){
    while(map[array[i]]!.count<((parent(i)?.count)??-1)){
      swap(i,(i/2).floor());
      i=(i/2).floor();
    }
  }
  swap(int a,int b){
    String i=array[a];
    array[a]=array[b];
    array[b]=i;
  }
  keyCount? lchild(int i){
    if(array.length<=i*2)return null;
    return map[array[i*2]];
  }
  keyCount? rchild(int i){
    if(array.length<=i*2+1)return null;
    return map[array[i*2+1]];
  }
  keyCount? parent(int i){
    if(i<2)return null;
    return map[array[(i/2).floor()]];
  }


}

class keyCount{
  late int count;
  late Map<String,dynamic> data;
  keyCount({this.count=1,required this.data});

  add1(){
    count++;
  }

  @override
  String toString() {
    return "count:${count},data:${data}";
  }


  /// todo:here need to be changed as well
  Map<String,dynamic> toMap(){
    return {"count":count,"data":data};
  }

  /// todo:here need to be changed as well
  keyCount.formMap(Map<String,dynamic> m){
    count=m["count"];
    data=m["data"];
  }

}

==> data_cache-0.0.2%2B1.tar.gz: lib/cacher.dart <==

import 'dart:convert';
import 'dart:core';

import 'package:shared_preferences/shared_preferences.dart';

class cacheManager{
  static const inf="info";
  static const cas="cs";
  static const ts="ts";
  static const av="av";

  static const map="d";
  static const array="a";
  // static const lmt=50;
  static const array_N="d";
  static const array_ID="a";

  static const maxCommit=3;

  late Map<Type,dataCount> data;
  late SharedPreferences sp;
  Iterable<Type> get classesKeys=>converter.keys;
  final Map<Type,List<Function>> converter;

  int commits=0;
  cacheManager(this.converter){
    if(!converter.keys.contains(Map))converter[Map]=[(Map m)=>m,(Map m)=>m];
    SharedPreferences.getInstance().then((v){
      sp=v;
      classesKeys.forEach((c) {
        if(v.containsKey(c.toString())&&v.getString(c.toString())!=null) {
          data[c]=dataCount.fromMap(json.decode(v.getString(c.toString())!));
        }
      });
    });
    data=Map<Type,dataCount>.fromIterables(classesKeys,
        [for(var c in classesKeys)dataCount()]);
  }

  void incrementCommits(){
    commits++;
    if(commits==maxCommit){
      classesKeys.forEach((k) {
        if(data[k]!=null)
          sp.setString(k.toString(), json.encode(data[k]!.toMap()));});
      commits=0;
    }
  }

  void addOrUpdate<T>(T i){
    final Map<String,dynamic> r=converter[T]![0](i);
    data[T]!.insert(r["id"], r);
    incrementCommits();
  }
  void update(Type T,String key,Map<String,dynamic> i){
    data[T]!.modify(key, i);
    incrementCommits();
  }

  T? get<T>(String id){
    if(data[T]!.get(id)==null)return null;
    incrementCommits();
    return converter[T]![1](data[T]!.get(id));
  }


  void check(){
    print("cacheManager instance:\n");
    data.forEach((key, value) {
      print(key);
      value.check();
    });
    print("end check\n=============================\n\n");
  }
}

class dataCount{
  static const lmt=6;
  static const int intMaxValue = 9007199254740991;
  int count=0;
  // dynamic data;
  final List<String> array;//=<String>["0"];
  final Map<String,keyCount> map;//=<String,keyCount<T>>{};
  dataCount():array=<String>["0"],map=<String,keyCount>{};

  void check(){
    print("  usage rank:");
    array.forEach((e) {
      print("    ${e} times :${map[e].toString()}");
    });
    print("  data map:");
    map.forEach((key, value) {
      print("    key:${key}");
      print("    data:${value.toString()}");
    });
  }

  Map<String,dynamic> toMap(){
    final mapm={};
    map.forEach((k, v) {mapm[k]= v.toMap();});
    return {"array":array,"map":mapm};
  }

  dataCount.fromMap(Map<String,dynamic> m):
        array=m["array"].cast<String>(),
        map=(m["map"] as Map).map((k, v) => MapEntry(k, keyCount.formMap(v)));

  Map<String,dynamic>? get(String key){
    final r=map[key];
    if(r!=null) insert(key, r.data);
    return r?.data;
  }

  ///if(key already exist) count+1,also set data to newdata
  ///if(key not exist)
  ///   if(not full) insert keyCount(data: data,count: 1)
  ///   if(full) insert keyCount(data: data,count: minUseCount+1)
  void insert(String key,Map<String,dynamic> data){
    if(map.containsKey(key)){ //// add count by 1 then bubble down
      int i=1;
      while(array[i]!=key){i++;}
      map[key]!.add1();
      map[array[i]]!.data=data;
      bubbledown(i);
    }else if(array.length<lmt){ /// add to bottom then bubble up
      map[key]=keyCount(data: data);
      int i=array.length;
      array.add(key);
      bubbleup(i);
    }else{
      final oldKey=array[1];
      map[key]=keyCount(data: data,count: map[oldKey]!.count+1);
      array[1]=key;
      map.remove(oldKey);
      bubbledown(1);
    }
  }

  ///modify data,without adding count
  void modify(String key,Map<String,dynamic> change){
    if(map.containsKey(key)){
      map[key]!.data=map[key]!.data..addAll(change);
      insert(key, map[key]!.data);
    }
  }

  bubbledown(int i){
    int l=((lchild(i)?.count)??(intMaxValue));
    int r=((rchild(i)?.count)??(intMaxValue));
    while(map[array[i]]!.count>l||map[array[i]]!.count>r){
      if(r>l){
        swap(i, i*2);
        i=i*2;
      }
      else{
        swap(i, i*2+1);
        i=i*2+1;
      }
      l=((lchild(i)?.count)??(intMaxValue));
      r=((rchild(i)?.count)??(intMaxValue));
    }
  }

  bubbleup(int i){
    while(map[array[i]]!.count<((parent(i)?.count)??-1)){
      swap(i,(i/2).floor());
      i=(i/2).floor();
    }
  }
  swap(int a,int b){
    String i=array[a];
    array[a]=array[b];
    array[b]=i;
  }
  keyCount? lchild(int i){
    if(array.length<=i*2)return null;
    return map[array[i*2]];
  }
  keyCount? rchild(int i){
    if(array.length<=i*2+1)return null;
    return map[array[i*2+1]];
  }
  keyCount? parent(int i){
    if(i<2)return null;
    return map[array[(i/2).floor()]];
  }


}

class keyCount{
  late int count;
  late Map<String,dynamic> data;
  keyCount({this.count=1,required Map<String,dynamic> data}):this.data=data.cast<String,dynamic>();

  add1(){
    count++;
  }

  @override
  String toString() {
    return "count:${count},data:${data}";
  }


  /// todo:here need to be changed as well
  Map<String,dynamic> toMap(){
    return {"count":count,"data":data};
  }

  /// todo:here need to be changed as well
  keyCount.formMap(Map<String,dynamic> m){
    count=m["count"];
    data=m["data"].cast<String,dynamic>();
  }

}
